@@ -54,6 +54,7 @@ class BookmarkDialog(QDialog):
         self.bookmarkCombo.setEditable(True)
         self.bookmarkCombo.currentIndexChanged.connect(self.bookmarkTextChanged)
         self.bookmarkCombo.editTextChanged.connect(self.bookmarkTextChanged)
+        qtlib.allowCaseChangingInput(self.bookmarkCombo)
         form.addRow(_('Bookmark:'), self.bookmarkCombo)
 
         ### Rename input
@@ -61,6 +62,15 @@ class BookmarkDialog(QDialog):
         self.newNameEdit.textEdited.connect(self.bookmarkTextChanged)
         form.addRow(_('New Name:'), self.newNameEdit)
 
+        ### Activate checkbox
+        self.activateCheckBox = QCheckBox()
+        if self.node == self.repo['.'].node():
+            self.activateCheckBox.setChecked(True)
+        else:
+            self.activateCheckBox.setChecked(False)
+            self.activateCheckBox.setEnabled(False)
+        form.addRow(_('Activate:'), self.activateCheckBox)
+
         ## bottom buttons
         BB = QDialogButtonBox
         bbox = QDialogButtonBox()
@@ -109,7 +119,6 @@ class BookmarkDialog(QDialog):
         # add bookmarks to drop-down list
         marks = self.repo._bookmarks.keys()[:]
         marks.sort()
-        marks.reverse()
         cur = self.bookmarkCombo.currentText()
         self.bookmarkCombo.clear()
         for bookmark in marks:
@@ -164,8 +173,10 @@ class BookmarkDialog(QDialog):
         def finished():
             self.set_status(_("Bookmark '%s' has been added") % bookmark, True)
 
-        cmdline = ['bookmark', '--repository', self.repo.root,
-                   '--rev', str(self.rev), bookmarklocal]
+        cmdline = ['bookmarks', '--repository', self.repo.root]
+        if not self.activateCheckBox.isChecked():
+            cmdline.extend(['--rev', str(self.rev)])
+        cmdline.append(bookmarklocal)
         self.cmd.run(cmdline)
         self.finishfunc = finished
 
@@ -180,8 +191,10 @@ class BookmarkDialog(QDialog):
         def finished():
             self.set_status(_("Bookmark '%s' has been moved") % bookmark, True)
 
-        cmdline = ['bookmark', '--repository', self.repo.root,
-                   '--rev', str(self.rev), '--force', bookmarklocal]
+        cmdline = ['bookmarks', '--repository', self.repo.root]
+        if not self.activateCheckBox.isChecked():
+            cmdline.extend(['--rev', str(self.rev)])
+        cmdline.extend(['--force', bookmarklocal])
         self.cmd.run(cmdline)
         self.finishfunc = finished
 
@@ -195,7 +208,7 @@ class BookmarkDialog(QDialog):
         def finished():
             self.set_status(_("Bookmark '%s' has been removed") % bookmark, True)
 
-        cmdline = ['bookmark', '--repository', self.repo.root,
+        cmdline = ['bookmarks', '--repository', self.repo.root,
                    '--delete', bookmarklocal]
         self.cmd.run(cmdline)
         self.finishfunc = finished
@@ -219,7 +232,7 @@ class BookmarkDialog(QDialog):
             self.set_status(_("Bookmark '%s' has been renamed to '%s'") %
                             (name, newname), True)
 
-        cmdline = ['bookmark', '--repository', self.repo.root,
+        cmdline = ['bookmarks', '--repository', self.repo.root,
                    '--rename', namelocal, newnamelocal]
         self.cmd.run(cmdline)
         self.finishfunc = finished
