@@ -5,11 +5,10 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+import os
 import re
 
-from PyQt4 import Qsci
-from PyQt4.QtGui import *
-
+from PyQt4 import Qsci, QtGui
 from tortoisehg.hgqt import qtlib
 
 class _LexerSelector(object):
@@ -81,7 +80,8 @@ class LuaLexerSelector(_ScriptLexerSelector):
     regex = None
 
 class CppLexerSelector(_FilenameLexerSelector):
-    extensions = ('.c', '.cpp', '.cxx', '.cl', '.cu', '.h', '.hpp', '.hxx')
+    extensions = ('.c', '.cpp', '.cc', '.cxx', '.cl', '.cu',
+                  '.h', '.hpp', '.hh', '.hxx')
     _lexer = Qsci.QsciLexerCPP
 
 class DLexerSelector(_FilenameLexerSelector):
@@ -189,18 +189,16 @@ class DiffLexerSelector(_ScriptLexerSelector):
     _lexer = Qsci.QsciLexerDiff
     regex = re.compile(r'^@@ [-]\d+,\d+ [+]\d+,\d+ @@$')
     def cfg_lexer(self, lexer):
-        #lexer.setDefaultPaper(QtGui.QColor(cfg.getDiffBGColor()))
-        #lexer.setColor(QtGui.QColor(cfg.getDiffFGColor()), -1)
         for label, i in (('diff.inserted', 6),
                          ('diff.deleted', 5),
                          ('diff.hunk', 4)):
             effect = qtlib.geteffect(label)
             for e in effect.split(';'):
                 if e.startswith('color:'):
-                    lexer.setColor(QColor(e[7:]), i)
+                    lexer.setColor(QtGui.QColor(e[7:]), i)
                 if e.startswith('background-color:'):
                     lexer.setEolFill(True, i)
-                    lexer.setPaper(QColor(e[18:]), i)
+                    lexer.setPaper(QtGui.QColor(e[18:]), i)
         font = qtlib.getfont('fontdiff').font()
         lexer.setFont(font, -1)
         return lexer
@@ -211,13 +209,20 @@ for clsname, cls in globals().items():
     if clsname.startswith('_'):
         continue
     if isinstance(cls, type) and issubclass(cls, _LexerSelector):
-        #print clsname
         lexers.append(cls())
 
-def get_diff_lexer(parent):
+def difflexer(parent):
     return DiffLexerSelector().lexer(parent)
 
-def get_lexer(filename, filedata, parent):
+def getlexer(ui, filename, filedata, parent):
+    _, ext = os.path.splitext(filename)
+    if ext and len(ext) > 1:
+        ext = ext.lower()[1:]
+        pref = ui.config('thg-lexer', ext)
+        if pref:
+            lexer = getattr(Qsci, 'QsciLexer' + pref)
+            if lexer and isinstance(lexer, type):
+                return lexer(parent)
     for lselector in lexers:
         if lselector.match(filename, filedata):
             return lselector.lexer(parent)
