@@ -130,8 +130,20 @@ class Scintilla(QsciScintilla):
         self.setWrapVisualFlags(QsciScintilla.WrapFlagByBorder)
         self.textChanged.connect(self._resetfindcond)
         self._resetfindcond()
+        self.highlightLines = set()
+        self._setMultipleSelectionOptions()
         unbindConflictedKeys(self)
 
+    def _setMultipleSelectionOptions(self):
+        if hasattr(QsciScintilla, 'SCI_SETMULTIPLESELECTION'):
+            self.SendScintilla(QsciScintilla.SCI_SETMULTIPLESELECTION, True)
+            self.SendScintilla(QsciScintilla.SCI_SETADDITIONALSELECTIONTYPING,
+                               True)
+            self.SendScintilla(QsciScintilla.SCI_SETMULTIPASTE,
+                               QsciScintilla.SC_MULTIPASTE_EACH)
+            self.SendScintilla(QsciScintilla.SCI_SETVIRTUALSPACEOPTIONS,
+                               QsciScintilla.SCVS_RECTANGULARSELECTION)
+
     def read(self, f):
         result = super(Scintilla, self).read(f)
         self.setDefaultEolMode()
@@ -352,12 +364,15 @@ class Scintilla(QsciScintilla):
         for m in pat.finditer(unicode(self.text()).encode('utf-8')):
             self.SendScintilla(self.SCI_INDICATORFILLRANGE,
                                m.start(), m.end() - m.start())
+            line = self.lineIndexFromPosition(m.start())[0]
+            self.highlightLines.add(line)
 
     @pyqtSlot()
     def clearHighlightText(self):
         self.SendScintilla(self.SCI_SETINDICATORCURRENT,
                            self._highlightIndicator)
         self.SendScintilla(self.SCI_INDICATORCLEARRANGE, 0, self.length())
+        self.highlightLines.clear()
 
     @util.propertycache
     def _highlightIndicator(self):
@@ -417,24 +432,29 @@ class Scintilla(QsciScintilla):
 
     if not hasattr(QsciScintilla, 'setIndicatorForegroundColor'):
         def setIndicatorForegroundColor(self, color, indicatorNumber):
-            self.SendScintilla(self.SCI_INDICSETFORE, indicatorNumber, color);
-            self.SendScintilla(self.SCI_INDICSETALPHA, indicatorNumber, color.alpha());
+            self.SendScintilla(self.SCI_INDICSETFORE, indicatorNumber, color)
+            self.SendScintilla(self.SCI_INDICSETALPHA, indicatorNumber,
+                               color.alpha())
 
     if not hasattr(QsciScintilla, 'clearIndicatorRange'):
-        def clearIndicatorRange(self, lineFrom, indexFrom, lineTo, indexTo, indicatorNumber):
+        def clearIndicatorRange(self, lineFrom, indexFrom, lineTo, indexTo,
+                                indicatorNumber):
             start = self.positionFromLineIndex(lineFrom, indexFrom)
             finish = self.positionFromLineIndex(lineTo, indexTo)
 
-            self.SendScintilla(self.SCI_SETINDICATORCURRENT, indicatorNumber);
-            self.SendScintilla(self.SCI_INDICATORCLEARRANGE, start, finish - start);
+            self.SendScintilla(self.SCI_SETINDICATORCURRENT, indicatorNumber)
+            self.SendScintilla(self.SCI_INDICATORCLEARRANGE,
+                               start, finish - start)
 
     if not hasattr(QsciScintilla, 'fillIndicatorRange'):
-        def fillIndicatorRange(self, lineFrom, indexFrom, lineTo, indexTo, indicatorNumber):
+        def fillIndicatorRange(self, lineFrom, indexFrom, lineTo, indexTo,
+                               indicatorNumber):
             start = self.positionFromLineIndex(lineFrom, indexFrom)
             finish = self.positionFromLineIndex(lineTo, indexTo)
 
-            self.SendScintilla(self.SCI_SETINDICATORCURRENT, indicatorNumber);
-            self.SendScintilla(self.SCI_INDICATORFILLRANGE, start, finish - start);
+            self.SendScintilla(self.SCI_SETINDICATORCURRENT, indicatorNumber)
+            self.SendScintilla(self.SCI_INDICATORFILLRANGE,
+                               start, finish - start)
 
 
 class SearchToolBar(QToolBar):
@@ -643,7 +663,9 @@ def findTabIndentsInLines(lines, linestocheck=100):
 def fileEditor(filename, **opts):
     'Open a simple modal file editing dialog'
     dialog = QDialog()
-    dialog.setWindowFlags(dialog.windowFlags() & ~Qt.WindowContextHelpButtonHint | Qt.WindowMaximizeButtonHint)
+    dialog.setWindowFlags(dialog.windowFlags()
+                          & ~Qt.WindowContextHelpButtonHint
+                          | Qt.WindowMaximizeButtonHint)
     dialog.setWindowTitle(filename)
     dialog.setLayout(QVBoxLayout())
     editor = Scintilla()
@@ -661,6 +683,9 @@ def fileEditor(filename, **opts):
     searchbar.conditionChanged.connect(editor.highlightText)
     searchbar.hide()
     def showsearchbar():
+        text = editor.selectedText()
+        if text:
+            searchbar.setPattern(text)
         searchbar.show()
         searchbar.setFocus(Qt.OtherFocusReason)
     qtlib.newshortcutsforstdkey(QKeySequence.Find, dialog, showsearchbar)
