@@ -14,7 +14,7 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import binascii
+import binascii, os, re
 
 from mercurial import util, error
 from mercurial.util import propertycache
@@ -23,6 +23,7 @@ from mercurial.context import workingctx
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.graph import Graph
 from tortoisehg.hgqt.graph import revision_grapher
+from tortoisehg.hgqt.graph import LINE_TYPE_GRAFT
 from tortoisehg.hgqt import qtlib
 from tortoisehg.hgqt.qreorder import writeSeries
 
@@ -48,6 +49,7 @@ COLUMNHEADERS = (
     ('Description', _('Description', 'column header')),
     ('Author', _('Author', 'column header')),
     ('Tags', _('Tags', 'column header')),
+    ('Latest tags', _('Latest tags', 'column header')),
     ('Node', _('Node', 'column header')),
     ('Age', _('Age', 'column header')),
     ('LocalTime', _('Local Time', 'column header')),
@@ -58,6 +60,7 @@ COLUMNHEADERS = (
     )
 
 UNAPPLIED_PATCH_COLOR = '#999999'
+HIDDENREV_COLOR = '#666666'
 
 def get_color(n, ignore=()):
     """
@@ -70,6 +73,56 @@ def get_color(n, ignore=()):
         colors = COLORS
     return colors[n % len(colors)]
 
+def get_style(line_type):
+    if line_type == LINE_TYPE_GRAFT:
+        return Qt.DashLine
+    return Qt.SolidLine
+
+def get_width(line_type):
+    if line_type == LINE_TYPE_GRAFT:
+        return 1
+    return 2
+
+def _parsebranchcolors(value):
+    r"""Parse tortoisehg.branchcolors setting
+
+    >>> _parsebranchcolors('foo:#123456  bar:#789abc ')
+    [('foo', '#123456'), ('bar', '#789abc')]
+    >>> _parsebranchcolors(r'foo\ bar:black foo\:bar:white')
+    [('foo bar', 'black'), ('foo:bar', 'white')]
+
+    >>> _parsebranchcolors(r'\u00c0:black')
+    [('\xc0', 'black')]
+    >>> _parsebranchcolors('\xc0:black')
+    [('\xc0', 'black')]
+
+    >>> _parsebranchcolors(None)
+    []
+    >>> _parsebranchcolors('ill:formed:value no-value')
+    []
+    >>> _parsebranchcolors(r'\ubad:unicode-repr')
+    []
+    """
+    if not value:
+        return []
+
+    colors = []
+    for e in re.split(r'(?:(?<=\\\\)|(?<!\\)) ', value):
+        pair = re.split(r'(?:(?<=\\\\)|(?<!\\)):', e)
+        if len(pair) != 2:
+            continue # ignore ill-formed
+        key, val = pair
+        key = key.replace('\\:', ':').replace('\\ ', ' ')
+        if r'\u' in key:
+            # apply unicode_escape only if \u found, so that raw non-ascii
+            # value isn't always mangled.
+            try:
+                key = hglib.fromunicode(key.decode('unicode_escape'))
+            except (UnicodeDecodeError, UnicodeEncodeError):
+                continue
+        colors.append((key, val))
+    return colors
+
 class HgRepoListModel(QAbstractTableModel):
     """
     Model used for displaying the revisions of a Hg *local* repository
@@ -82,10 +135,13 @@ class HgRepoListModel(QAbstractTableModel):
     _allcolnames = dict(COLUMNHEADERS)
 
     _columns = ('Graph', 'Rev', 'Branch', 'Description', 'Author', 'Age', 'Tags', 'Phase',)
+    _columnfonts = {'Node': QFont("Monospace"),
+                    'Converted': QFont("Monospace")}
     _stretchs = {'Description': 1, }
     _mqtags = ('qbase', 'qtip', 'qparent')
 
-    def __init__(self, repo, cfgname, branch, revset, rfilter, parent):
+    def __init__(self, repo, cfgname, branch, revset, rfilter, parent,
+            showhidden=False, allparents=False):
         """
         repo is a hg repo instance
         """
@@ -102,6 +158,11 @@ class HgRepoListModel(QAbstractTableModel):
         self.unicodestar = True
         self.unicodexinabox = True
         self.cfgname = cfgname
+        self.latesttags = {-1: 'null'}
+        self.fullauthorname = False
+        self.filterbranch = branch  # unicode
+        self.showhidden = showhidden
+        self.allparents = allparents
 
         # To be deleted
         self._user_colors = {}
@@ -111,7 +172,7 @@ class HgRepoListModel(QAbstractTableModel):
             self.initBranchColors()
             self.reloadConfig()
             self.updateColumns()
-            self.setBranch(branch)
+            self._initGraph()
 
     def initBranchColors(self):
         # Set all the branch colors once on a fixed order,
@@ -119,35 +180,54 @@ class HgRepoListModel(QAbstractTableModel):
 
         # Always assign the first color to the default branch
         self.namedbranch_color('default')
+
+        # Set the colors specified in the tortoisehg.brachcolors config key
+        self._branch_colors.update(_parsebranchcolors(
+            self.repo.ui.config('tortoisehg', 'branchcolors')))
+
         # Then assign colors to all branches in alphabetical order
         # Note that re-assigning the color to the default branch
         # is not expensive
         for branch in sorted(self.repo.branchtags().keys()):
             self.namedbranch_color(branch)
 
-    def setBranch(self, branch=None, allparents=True):
-        self.filterbranch = branch  # unicode
+    def setBranch(self, branch, allparents=False):
+        self.filterbranch = branch
+        self.allparents = allparents
+        self._initGraph()
+
+    def setShowHidden(self, visible):
+        self.showhidden = visible
+        self._initGraph()
+
+    def _initGraph(self):
+        branch = self.filterbranch
+        allparents = self.allparents
+        showhidden = self.showhidden
         self.invalidateCache()
         if self.revset and self.filterbyrevset:
             grapher = revision_grapher(self.repo,
                                        branch=hglib.fromunicode(branch),
-                                       revset=self.revset)
+                                       revset=self.revset,
+                                       showhidden=showhidden)
             self.graph = Graph(self.repo, grapher, include_mq=False)
         else:
             grapher = revision_grapher(self.repo,
                                        branch=hglib.fromunicode(branch),
-                                       allparents=allparents)
+                                       allparents=allparents,
+                                       showhidden=showhidden)
             self.graph = Graph(self.repo, grapher, include_mq=True)
         self.rowcount = 0
         self.layoutChanged.emit()
         self.ensureBuilt(row=0)
         self.showMessage.emit('')
-        QTimer.singleShot(0, lambda: self.filled.emit())
+        QTimer.singleShot(0, self, SIGNAL('filled()'))
 
     def reloadConfig(self):
         _ui = self.repo.ui
         self.fill_step = int(_ui.config('tortoisehg', 'graphlimit', 500))
         self.authorcolor = _ui.configbool('tortoisehg', 'authorcolor')
+        self.fullauthorname = _ui.configbool('tortoisehg', 'fullauthorname')
 
     def updateColumns(self):
         s = QSettings()
@@ -254,7 +334,7 @@ class HgRepoListModel(QAbstractTableModel):
             return '8' * 12 + '+'
         if column in ('LocalTime', 'UTCTime'):
             return hglib.displaytime(util.makedate())
-        if column == 'Tags':
+        if column in ('Tags', 'Latest tags'):
             try:
                 return sorted(self.repo.tags().keys(), key=lambda x: len(x))[-1][:10]
             except IndexError:
@@ -322,10 +402,11 @@ class HgRepoListModel(QAbstractTableModel):
             ymid = (y1 + y4)/2
             y3 = y1 + 3 * (y4 - y1)/4
 
-            for start, end, color in lines:
+            for start, end, color, line_type in lines:
                 lpen = QPen(pen)
                 lpen.setColor(QColor(get_color(color)))
-                lpen.setWidth(2)
+                lpen.setStyle(get_style(line_type))
+                lpen.setWidth(get_width(line_type))
                 painter.setPen(lpen)
                 x1 = self.col2x(start)
                 x2 = self.col2x(end)
@@ -343,6 +424,7 @@ class HgRepoListModel(QAbstractTableModel):
         dot_color = QColor(self.namedbranch_color(ctx.branch()))
         dotcolor = dot_color.lighter()
         pencolor = dot_color.darker()
+        truewhite = QColor("white")
         white = QColor("white")
         fillcolor = gnode.rev is None and white or dotcolor
 
@@ -372,26 +454,43 @@ class HgRepoListModel(QAbstractTableModel):
                               QPointF(centre_x - r, centre_y),])
             painter.drawPolygon(poly)
 
+        hiddenrev = ctx.hidden()
+        if hiddenrev:
+            painter.setBrush(truewhite)
+            white.setAlpha(64)
+            fillcolor.setAlpha(64)
         if ctx.thgmqappliedpatch():  # diamonds for patches
+            symbolsize = radius / 1.5
+            if hiddenrev:
+                diamond(symbolsize)
             if ctx.thgwdparent():
                 painter.setBrush(white)
-                diamond(2 * 0.9 * radius / 1.5)
+                diamond(2 * 0.9 * symbolsize)
             painter.setBrush(fillcolor)
-            diamond(radius / 1.5)
+            diamond(symbolsize)
         elif ctx.thgmqunappliedpatch():
+            symbolsize = radius / 1.5
+            if hiddenrev:
+                diamond(symbolsize)
             patchcolor = QColor('#dddddd')
             painter.setBrush(patchcolor)
             painter.setPen(patchcolor)
-            diamond(radius / 1.5)
+            diamond(symbolsize)
         elif ctx.extra().get('close'):
+            symbolsize = 0.5 * radius
+            if hiddenrev:
+                closesymbol(symbolsize)
             painter.setBrush(fillcolor)
-            closesymbol(0.5 * radius)
+            closesymbol(symbolsize)
         else:  # circles for normal revisions
+            symbolsize = 0.5 * radius
+            if hiddenrev:
+                circle(symbolsize)
             if ctx.thgwdparent():
                 painter.setBrush(white)
                 circle(0.9 * radius)
             painter.setBrush(fillcolor)
-            circle(0.5 * radius)
+            circle(symbolsize)
 
     def invalidateCache(self):
         self._cache = []
@@ -408,11 +507,19 @@ class HgRepoListModel(QAbstractTableModel):
     def data(self, index, role):
         if not index.isValid():
             return nullvariant
+        # font is not cached in self._cache since it is equal for all rows
+        if role == Qt.FontRole:
+            column = self._columns[index.column()]
+            return self._columnfonts.get(column, nullvariant)
         if role not in self._roleoffsets:
             return nullvariant
+        # repo may be changed while reading in case of postpull=rebase for
+        # example, and result in RevlogError. (issue #429)
         try:
             return self.safedata(index, role)
-        except Exception, e:
+        except error.RevlogError, e:
+            if 'THGDEBUG' in os.environ:
+                raise
             if role == Qt.DisplayRole:
                 return QVariant(hglib.tounicode(str(e)))
             else:
@@ -462,6 +569,8 @@ class HgRepoListModel(QAbstractTableModel):
         elif role == Qt.ForegroundRole:
             if ctx.thgmqunappliedpatch():
                 return QColor(UNAPPLIED_PATCH_COLOR)
+            if ctx.hidden():
+                return QColor(HIDDENREV_COLOR)
             if column == 'Author':
                 if self.authorcolor:
                     return QVariant(QColor(self.user_color(ctx.user())))
@@ -531,8 +640,6 @@ class HgRepoListModel(QAbstractTableModel):
         if orientation == Qt.Horizontal:
             if role == Qt.DisplayRole:
                 return QVariant(self._allcolnames[self._columns[section]])
-            if role == Qt.TextAlignmentRole:
-                return QVariant(Qt.AlignLeft)
         return nullvariant
 
     def rowFromRev(self, rev):
@@ -565,6 +672,33 @@ class HgRepoListModel(QAbstractTableModel):
                 b += u'--'
         return b
 
+    def getlatesttags(self, ctx, gnode):
+        rev = ctx.rev()
+        todo = [rev]
+        repo = self.repo
+        while todo:
+            rev = todo.pop()
+            if rev in self.latesttags:
+                continue
+            ctx = repo[rev]
+            tags = [t for t in ctx.tags() if repo.tagtype(t) == 'global']
+            if tags:
+                self.latesttags[rev] = ':'.join(sorted(tags))
+                continue
+            try:
+                if (ctx.parents()):
+                    ptag = max(
+                        self.latesttags[p.rev()] for p in ctx.parents())
+                else:
+                    ptag = ""
+            except KeyError:
+                # Cache miss - recurse
+                todo.append(rev)
+                todo.extend(p.rev() for p in ctx.parents())
+                continue
+            self.latesttags[rev] = ptag
+        return self.latesttags[rev]
+
     def gettags(self, ctx, gnode):
         if ctx.rev() is None:
             return ''
@@ -582,7 +716,10 @@ class HgRepoListModel(QAbstractTableModel):
 
     def getauthor(self, ctx, gnode):
         try:
-            return hglib.username(ctx.user())
+            user = ctx.user()
+            if not self.fullauthorname:
+                user = hglib.username(user)
+            return user
         except error.Abort:
             return _('Mercurial User')
 
@@ -610,6 +747,8 @@ class HgRepoListModel(QAbstractTableModel):
             # qtlib.markup(msg, fg=UNAPPLIED_PATCH_COLOR)
             msg = qtlib.markup(msg)
             return hglib.tounicode(text + ' ') + msg
+        if ctx.hidden():
+            return qtlib.markup(msg, fg=HIDDENREV_COLOR)
 
         parts = []
         if ctx.thgbranchhead():
@@ -663,7 +802,8 @@ class HgRepoListModel(QAbstractTableModel):
 
     def getconv(self, ctx, gnode):
         if ctx.rev() is not None:
-            cvt = ctx.extra().get('convert_revision', '')
+            extra = ctx.extra()
+            cvt = extra.get('convert_revision', '')
             if cvt:
                 if cvt.startswith('svn:'):
                     return cvt.split('@')[-1]
@@ -693,6 +833,7 @@ class HgRepoListModel(QAbstractTableModel):
         'Description': getlog,
         'Author':   getauthor,
         'Tags':     gettags,
+        'Latest tags':     getlatesttags,
         'Branch':   getbranch,
         'Filename': lambda self, ctx, gnode: gnode.extra[0],
         'Age':      lambda self, ctx, gnode: hglib.age(ctx.date()).decode('utf-8'),
