@@ -226,10 +226,12 @@ class Workbench(QMainWindow):
 
         self.actionGroupTaskView = QActionGroup(self)
         self.actionGroupTaskView.triggered.connect(self.onSwitchRepoTaskTab)
-        def addtaskview(icon, label, name):
+        def addtaskview(icon, label, name, shortcut=None):
             a = newaction(label, icon=None, checkable=True, data=name,
                           enabled='repoopen', menu='view')
             a.setIcon(qtlib.geticon(icon))
+            if shortcut:
+                a.setShortcut(shortcut)
             self.actionGroupTaskView.addAction(a)
             self.tasktbar.addAction(a)
             return a
@@ -253,13 +255,14 @@ class Workbench(QMainWindow):
         self.actionSelectTaskMQ = None
         self.actionSelectTaskPbranch = None
 
-        for taskname in tasklist:
+        for i, taskname in enumerate(tasklist):
             taskname = taskname.strip()
             taskinfo = taskdefs.get(taskname, None)
             if taskinfo is None:
                 newseparator(toolbar='task')
                 continue
-            tbar = addtaskview(taskinfo[0], taskinfo[1], taskname)
+            tbar = addtaskview(taskinfo[0], taskinfo[1], taskname,
+                               "Alt+%d" % (i + 1))
             if taskname == 'mq':
                 self.actionSelectTaskMQ = tbar
             elif taskname == 'pbranch':
@@ -354,54 +357,79 @@ class Workbench(QMainWindow):
         menu = QMenu(_('&Workbench Toolbars'), self)
         menu.addAction(self.edittbar.toggleViewAction())
         menu.addAction(self.docktbar.toggleViewAction())
-        menu.addAction(self.synctbar.toggleViewAction())
         menu.addAction(self.tasktbar.toggleViewAction())
+        menu.addAction(self.synctbar.toggleViewAction())
         menu.addAction(self.customtbar.toggleViewAction())
         self.menuView.addMenu(menu)
 
         newaction(_('Incoming'), data='incoming', icon='hg-incoming',
-                  tooltip=_('Check for incoming changes from selected URL'),
                   enabled='repoopen', toolbar='sync')
         newaction(_('Pull'), data='pull', icon='hg-pull',
-                  tooltip=_('Pull incoming changes from selected URL'),
                   enabled='repoopen', toolbar='sync')
         newaction(_('Outgoing'), data='outgoing', icon='hg-outgoing',
-                  tooltip=_('Detect outgoing changes to selected URL'),
                   enabled='repoopen', toolbar='sync')
         newaction(_('Push'), data='push', icon='hg-push',
-                  tooltip=_('Push outgoing changes to selected URL'),
                   enabled='repoopen', toolbar='sync')
         self.urlCombo = QComboBox(self)
         self.urlCombo.setSizeAdjustPolicy(QComboBox.AdjustToContents)
+        self.urlCombo.currentIndexChanged.connect(self._updateSyncUrlToolTip)
         self.synctbar.addWidget(self.urlCombo)
         self.synctbar.actionTriggered.connect(self._runSyncAction)
 
         self.updateMenu()
 
     def _setupUrlCombo(self, repo):
-        'repository has been switched, fill urlCombo with URLs'
-        aliases = [hglib.tounicode(alias)
-                   for alias, path in repo.ui.configitems('paths')]
+        """repository has been switched, fill urlCombo with URLs"""
+        pathdict = dict((hglib.tounicode(alias), hglib.tounicode(path))
+                         for alias, path in repo.ui.configitems('paths'))
+        aliases = pathdict.keys()
+
+        # 1. Sort the list if aliases
+        aliases.sort()
+        # 2. Place the default alias at the top of the list
         if 'default' in aliases:
             aliases.remove('default')
             aliases.insert(0, 'default')
+        # 3. Make a list of paths that have a 'push path'
+        # note that the default path will be first (if it has a push path),
+        # followed by the other paths that have a push path, alphabetically
+        haspushaliases = [alias for alias in aliases
+                         if alias + '-push' in aliases]
+        # 4. Place the "-push" paths next to their "pull paths"
+        regularaliases = []
         for a in aliases[:]:
-            if a + '-push' in aliases:
-                # add foo,foo-push entry to top of menu
-                aliases.insert(0, (a, a + '-push'))
-                # move foo,foo-push individual entries to bottom
-                aliases.remove(a)
-                aliases.remove(a + '-push')
-                aliases.append(a)
-                aliases.append(a + '-push')
+            if a.endswith('-push'):
+                if a[:-len('-push')] in haspushaliases:
+                    continue
+            regularaliases.append(a)
+            if a in haspushaliases:
+                regularaliases.append(a + '-push')
+        # 5. Create the list of 'combined aliases'
+        combinedaliases = [(a, a + '-push') for a in haspushaliases]
+        # 6. Put the combined aliases first, followed by the regular aliases
+        aliases = combinedaliases + regularaliases
+        # 7. Ensure the first path is a default path (either a
+        # combined "default | default-push" path or a regular default path)
+        if not 'default-push' in aliases and 'default' in aliases:
+            aliases.remove('default')
+            aliases.insert(0, 'default')
 
+        self.urlCombo.blockSignals(True)
         self.urlCombo.clear()
-        for a in aliases:
+        for n, a in enumerate(aliases):
             # text, (pull-alias, push-alias)
             if isinstance(a, tuple):
-                self.urlCombo.addItem(u'\u2193 %s | %s \u2191' % a, a)
+                itemtext = u'\u2193 %s | %s \u2191' % a
+                itemdata = tuple(pathdict[alias] for alias in a)
+                tooltip = _('pull: %s\npush: %s') % itemdata
             else:
-                self.urlCombo.addItem(a, (a, a))
+                itemtext = a
+                itemdata = (pathdict[a], pathdict[a])
+                tooltip = pathdict[a]
+            self.urlCombo.addItem(itemtext, itemdata)
+            self.urlCombo.setItemData(n, tooltip, Qt.ToolTipRole)
+        self.urlCombo.blockSignals(False)
+        self._updateSyncUrlToolTip(self.urlCombo.currentIndex())
 
     #@pyqtSlot()
     def _setupUrlComboIfCurrent(self):
@@ -410,13 +438,48 @@ class Workbench(QMainWindow):
             self._setupUrlCombo(w.repo)
 
     def _syncUrlFor(self, op):
-        """Current URL alias for the given sync operation"""
+        """Current URL for the given sync operation"""
         urlindex = self.urlCombo.currentIndex()
         if urlindex < 0:
             return
         opindex = {'incoming': 0, 'pull': 0, 'outgoing': 1, 'push': 1}[op]
         return self.urlCombo.itemData(urlindex).toPyObject()[opindex]
 
+    @pyqtSlot(int)
+    def _updateSyncUrlToolTip(self, index):
+        self._updateUrlComboToolTip(index)
+        self._updateSyncActionToolTip(index)
+
+    def _updateUrlComboToolTip(self, index):
+        if not self.urlCombo.count():
+            tooltip = _('There are no configured sync paths.\n'
+                        'Open the Synchronize tab to configure them.')
+        else:
+            tooltip = self.urlCombo.itemData(index, Qt.ToolTipRole).toString()
+        self.urlCombo.setToolTip(tooltip)
+
+    def _updateSyncActionToolTip(self, index):
+        if index < 0:
+            tooltips = {
+                'incoming': _('Check for incoming changes'),
+                'pull':     _('Pull incoming changes'),
+                'outgoing': _('Detect outgoing changes'),
+                'push':     _('Push outgoing changes'),
+                }
+        else:
+            pullurl, pushurl = self.urlCombo.itemData(index).toPyObject()
+            tooltips = {
+                'incoming': _('Check for incoming changes from\n%s') % pullurl,
+                'pull':     _('Pull incoming changes from\n%s') % pullurl,
+                'outgoing': _('Detect outgoing changes to\n%s') % pushurl,
+                'push':     _('Push outgoing changes to\n%s') % pushurl,
+                }
+
+        for a in self.synctbar.actions():
+            op = str(a.data().toString())
+            if op in tooltips:
+                a.setToolTip(tooltips[op])
+
     def _setupCustomTools(self, ui):
         tools, toollist = hglib.tortoisehgtools(ui,
             selectedlocation='workbench.custom-toolbar')
@@ -629,13 +692,13 @@ class Workbench(QMainWindow):
     @pyqtSlot(QString)
     def showRepo(self, root):
         """Activate the repo tab or open it if not available [unicode]"""
-        root = hglib.fromunicode(root)
+        root = unicode(root)
         for i in xrange(self.repoTabsWidget.count()):
             w = self.repoTabsWidget.widget(i)
             if hglib.tounicode(w.repo.root) == os.path.normpath(root):
                 self.repoTabsWidget.setCurrentIndex(i)
                 return w
-        return self._openRepo(root, False)
+        return self._openRepo(hglib.fromunicode(root), False)
 
     @pyqtSlot(unicode, QString)
     def setRevsetFilter(self, path, filter):
@@ -747,7 +810,7 @@ class Workbench(QMainWindow):
         if widget.closeRepoWidget():
             w = self.repoTabsWidget.widget(index)
             try:
-                reporoot = w.repo.root
+                reporoot = hglib.tounicode(w.repo.root)
             except:
                 reporoot = ''
             self.repoTabsWidget.removeTab(index)
@@ -760,7 +823,7 @@ class Workbench(QMainWindow):
         if 0 <= index < tw.count():
             w = tw.widget(index)
             try:
-                reporoot = w.repo.root
+                reporoot = hglib.tounicode(w.repo.root)
             except:
                 reporoot = ''
             if w and w.closeRepoWidget():
@@ -1303,7 +1366,8 @@ def run(ui, *pats, **opts):
         root = hglib.tounicode(root)
         bundle = opts.get('bundle')
         if bundle:
-            w._openRepo(root, False, bundle=bundle)
+            w._openRepo(hglib.fromunicode(root), False,
+                        bundle=hglib.tounicode(bundle))
         else:
             w.showRepo(root)
 
